<?php
class bizextUser extends userModel
{
    public function getList($dept = 0, $mode = 'normal', $accountList = '', $search = '', $orderBy = 'id', $pager = null)
    {
        $deptList = array();
        if($dept)
        {
            $this->loadModel('tree');
            if(!is_array($dept)) $dept = explode(',', $dept);
            foreach($dept as $d)
            {
                $depts    = $this->tree->getFamily($d);
                $deptList = array_merge($deptList, $depts);
            }
        }

        return $this->dao->select('*')->from(TABLE_USER)
            ->where(1)
            ->beginIF($deptList)->andWhere('dept')->in($deptList)->fi()
            ->beginIF($accountList)->andWhere('account')->in($accountList)->fi()
            ->beginIF($mode != 'all')->andWhere('deleted')->eq('0')->fi()
            ->beginIF($mode == 'forbid')->andWhere('locked')->ge(helper::now())->fi()
            ->beginIF($mode == 'unofficial')->andWhere('unofficial')->eq('1')->fi()

            ->beginIF($mode == 'salary')
            ->andWhere('noSalary')->eq('0')
            ->andWhere('locked', true)->eq('0000-00-00 00:00:00')
            ->orWhere('locked')->lt(helper::now())
            ->markRight(1)
            ->fi()

            ->beginIF($mode == 'normal')
            ->andWhere('unofficial')->eq('0')
            ->andWhere('locked', true)->eq('0000-00-00 00:00:00')
            ->orWhere('locked')->lt(helper::now())
            ->markRight(1)
            ->fi()

            ->beginIF($search)
            ->andWhere('account', true)->like("%$search%")
            ->orWhere('realname')->like("%$search%")
            ->markRight(1)
            ->fi()

            ->orderBy($orderBy)
            ->page($pager)
            ->fetchAll();
    }

    public function getPairs($params = '', $dept = 0, $pager = null)
    {
        $users = $this->dao->select('account, realname')->from(TABLE_USER) 
            ->where(1)
            ->beginIF(strpos($params, 'nodeleted') !== false)->andWhere('deleted')->eq('0')->fi()
            ->beginIF(strpos($params, 'noforbidden') !== false)
            ->andWhere('locked', true)->eq('0000-00-00 00:00:00')
            ->orWhere('locked')->lt(helper::now())
            ->markRight(1)
            ->fi()
            ->beginIF(strpos($params, 'admin') !== false)->andWhere('admin')->ne('no')->fi()
            ->beginIF(strpos($params, 'unofficial') === false)->andWhere('unofficial')->eq('0')->fi()
            ->beginIF(strpos($params, 'hasSalary') !== false)->andWhere('noSalary')->eq('0')->fi()
            ->beginIF($dept)->andWhere('dept')->in($dept)->fi()
            ->orderBy('id_asc')    
            ->beginIF($pager)->page($pager)->fi()
            ->fetchPairs();

        foreach($users as $account => $realname) if($realname == '') $users[$account] = $account; 

        /* Append empty users. */
        if(strpos($params, 'noempty') === false) $users = array('' => '') + $users;
        if(strpos($params, 'noclosed') === false) $users = $users + array('closed' => 'Closed');

        return $users;
    }

    public function identify($account, $password)
    {
        $user = parent::identify($account, $password);

        if($user) return $user;
        if(empty($password)) return $user;

        $this->app->loadModuleConfig('ldap');
        if(empty($this->config->ldap)) return $user;
        $ldapConfig = $this->config->ldap;
        if(empty($ldapConfig->turnon)) return $user;
        $ldapConn = ldap_connect($ldapConfig->host, $ldapConfig->port);
        ldap_set_option($ldapConn, LDAP_OPT_PROTOCOL_VERSION, 3);
        ldap_set_option($ldapConn, LDAP_OPT_REFERRALS, 0);

        if(!isset($ldapConfig->anonymous)) @ldap_bind($ldapConn, $ldapConfig->admin, $ldapConfig->password);
        $ldapList = ldap_search($ldapConn, $ldapConfig->baseDN, "({$ldapConfig->account}=$account)");
        $infos    = ldap_get_entries($ldapConn, $ldapList);

        if(!isset($infos[0])) return $user;
        $info = $infos[0];
        if(empty($info['dn'])) return $user;
        if(@ldap_bind($ldapConn, $info['dn'], $password))
        {
            $user = $this->getByOpenID($account, 'ldap');
            if(!$user) $user = false;
        }

        ldap_free_result($ldapList);
        ldap_unbind($ldapConn);
        return $user;
    }

    public function getLDAPUser()
    {
        $users = array();
        $this->app->loadModuleConfig('ldap');
        if(!$this->config->ldap) die(js::locate(helper::createLink('ldap', 'index')));
        $ldapConfig = $this->config->ldap;
        if(!$ldapConfig->turnon) return 'off';
        $ldapConn = ldap_connect($ldapConfig->host, $ldapConfig->port);
        ldap_set_option($ldapConn, LDAP_OPT_PROTOCOL_VERSION, 3);
        ldap_set_option($ldapConn, LDAP_OPT_REFERRALS, 0);

        if(!isset($ldapConfig->anonymous)) @ldap_bind($ldapConn, $ldapConfig->admin, $ldapConfig->password);
        $ldapList  = ldap_search($ldapConn, "$ldapConfig->baseDN", "({$ldapConfig->account}=*)");
        $infos     = ldap_get_entries($ldapConn, $ldapList);
        $ldapUsers = $this->dao->select('openID')->from(TABLE_OAUTH)->where('provider')->eq('ldap')->fetchPairs();

        if($this->config->debug) file_put_contents($this->app->getTmpRoot() . 'log/ldap.log.php', "<?php\n die(); \n?>\n" . var_export($infos, true));

        foreach($infos as $key => $info)
        {
            if(!isset($info[$ldapConfig->account])) continue;

            $account = $info[$ldapConfig->account][0];
            if(empty($account)) continue;
            if(isset($ldapUsers[$account])) continue;

            $realname = $account;
            if(!empty($ldapConfig->realname) and !empty($info[strtolower($ldapConfig->realname)])) $realname = $info[strtolower($ldapConfig->realname)];

            $user['account']  = $account;
            $user['realname'] = $realname;
            $user['email']    = (empty($ldapConfig->email) or empty($info[$ldapConfig->email])) ? '' : $info[$ldapConfig->email];
            $user['mobile']   = (empty($ldapConfig->mobile) or empty($info[$ldapConfig->mobile])) ? '' : $info[$ldapConfig->mobile];
            $user['phone']    = (empty($ldapConfig->phone) or empty($info[$ldapConfig->phone])) ? '' : $info[$ldapConfig->phone];

            if(is_array($user['realname'])) $user['realname'] = $user['realname'][0];
            if(is_array($user['email']))    $user['email']    = $user['email'][0];
            if(is_array($user['phone']))    $user['phone']    = $user['phone'][0];
            if(is_array($user['mobile']))   $user['mobile']   = $user['mobile'][0];

            $users[$account] = $user;
        }
        ldap_free_result($ldapList);
        ldap_unbind($ldapConn);
        return $users;
    }

    /**
     * Check input when import ldap users. 
     * 
     * @access public
     * @return array
     */
    public function checkInput()
    {
        $errors = array();
        $users  = $this->dao->select('account, email')->from(TABLE_USER)->fetchPairs();

        foreach($this->post->add as $key)
        {
            if(!validater::checkLength($this->post->ldapAccount[$key], 60)) $errors["ldapAccount$key"] = sprintf($this->lang->user->error->length, $this->lang->user->ldapUser, 60);

            if(empty($this->post->createAccount[$key]))
            {
                if(!$this->post->account[$key]) $errors["account$key"] = sprintf($this->lang->error->notempty, $this->lang->user->zdooUser);
            }
            else
            {
                $newAccount = $this->post->newAccount[$key];
                $realname   = $this->post->realname[$key];
                $email      = $this->post->email[$key];

                if($realname) 
                {
                    if(!validater::checkLength($realname, 30)) $errors["realname$key"] = sprintf($this->lang->user->error->length, $this->lang->user->realname, 30);
                }
                else
                {
                    $errors["realname$key"] = sprintf($this->lang->error->notempty, $this->lang->user->realname);
                }

                if($email)
                {
                    if(!validater::checkEmail($email))      $errors["email$key"] = sprintf($this->lang->error->email, $this->lang->user->email);
                    if(!validater::checkLength($email, 90)) $errors["email$key"] = sprintf($this->lang->user->error->length, $this->lang->user->zdooUser, 90);
                    if(in_array($email, $users))            $errors["email$key"] = sprintf($this->lang->error->unique, $this->lang->user->email, $email);
                }
                else
                {
                    $errors["email$key"] = sprintf($this->lang->error->notempty, $this->lang->user->email);
                }

                if($newAccount)
                {
                    if(!validater::checkAccount($newAccount))    $errors["newAccount$key"] = sprintf($this->lang->error->account, $this->lang->user->zdooUser);
                    if(!validater::checkLength($newAccount, 30)) $errors["newAccount$key"] = sprintf($this->lang->user->error->length, $this->lang->user->zdooUser, 30);
                    if(isset($users[$newAccount]))               $errors["newAccount$key"] = sprintf($this->lang->error->unique, $this->lang->user->zdooUser, $newAccount);
                }
                else
                {
                    $errors["newAccount$key"] = sprintf($this->lang->error->notempty, $this->lang->user->zdooUser);
                }
            }
        }
        return $errors;
    }

    /**
     * Import users from ldap. 
     * 
     * @param  int    $userLeft
     * @access public
     * @return array | bool
     */
    public function importLDAP($userLeft)
    {
        if(!$this->post->add) return true;

        /* Check input. */
        $errors = $this->checkInput();
        if($errors) return array('result' => 'fail', 'message' => $errors);

        $gender    = 'u';
        $dept      = 0;
        $role      = '';
        $group     = 0;
        $ldapUsers = $this->getLDAPUser();
        
        $user = new stdclass();
        $user->join = helper::now();

        $userGroup = new stdclass();

        $oauth = new stdclass();
        $oauth->provider ='ldap';

        $newUser = 0;
        foreach($this->post->add as $key)
        {
            $ldapAccount = $this->post->ldapAccount[$key];
            if(!isset($ldapUsers[$ldapAccount])) continue;

            if($newUser >= $userLeft) continue;

            if(empty($this->post->createAccount[$key]))
            {
                $oauth->account = $this->post->account[$key];
            }
            else
            {
                $gender = $this->post->gender[$key] == 'ditto' ? $gender : $this->post->gender[$key];
                $dept   = $this->post->dept[$key]   == 'ditto' ? $dept   : $this->post->dept[$key];
                $role   = $this->post->role[$key]   == 'ditto' ? $role   : $this->post->role[$key];
                $group  = $this->post->group[$key]  == 'ditto' ? $group  : $this->post->group[$key];

                $user->account  = $this->post->newAccount[$key];
                $user->realname = $this->post->realname[$key];
                $user->gender   = $gender;
                $user->dept     = $dept;
                $user->role     = $role;
                $user->email    = $this->post->email[$key];

                $this->dao->insert(TABLE_USER)->data($user)->autoCheck()->exec();
                if(dao::isError()) continue;

                $newUser++;

                $userID = $this->dao->lastInsertId();
                $this->loadModel('action')->create('user', $userID, 'created');

                if($group)
                {
                    $userGroup->account = $this->post->newAccount[$key];
                    $userGroup->group   = $group;

                    $this->dao->replace(TABLE_USERGROUP)->data($userGroup)->exec();
                }

                $oauth->account = $this->post->newAccount[$key];
            }

            $oauth->openID = $ldapAccount;

            $this->dao->insert(TABLE_OAUTH)->data($oauth)->autoCheck()->exec();
        }

        return !dao::isError();
    }

    /**
     * Get user by account. 
     * 
     * @param  int    $account 
     * @access public
     * @return object
     */
    public function getByAccount($account)
    {
        $user = $this->dao->select('u.*, o.provider as provider, openID as openID')->from(TABLE_USER)->alias('u')
                ->leftJoin(TABLE_OAUTH)->alias('o')->on('u.account = o.account')
                ->beginIF(validater::checkEmail($account))->where('u.email')->eq($account)->fi()
                ->beginIF(!validater::checkEmail($account))->where('u.account')->eq($account)->fi()
                ->andWhere('deleted')->eq('0')
                ->fetch();
        $user->deptName = '';
        if(!empty($user->dept))
        {
            $dept           = $this->loadModel('tree')->getByID($user->dept, 'dept');
            $user->deptName = isset($dept->name) ? $dept->name : '';
        }
        $user->companyName = '';
        if(!empty($user->company))
        {
            $company           = $this->loadModel('company')->getByID($user->company);
            $user->companyName = isset($company->name) ? $company->name : '';
        }
        return $user;
    }

    /**
     * Get user by openID.
     * 
     * @param  int    $openID 
     * @access public
     * @return void
     */
    public function getByOpenID($openID, $provider)
    {
        return $this->dao->select('u.*, o.provider as provider, o.openID as openID')->from(TABLE_USER)->alias('u')
            ->leftJoin(TABLE_OAUTH)->alias('o')->on('u.account = o.account')
            ->where('o.provider')->eq($provider)
            ->andWhere('o.openID')->eq($openID)
            ->fetch();
    }

    /**
     * Get openID pairs.
     * 
     * @param  string  $account 
     * @access public
     * @return void
     */
    public function getOpenIDPairs($provider, $account = '')
    {
        return $this->dao->select('account, openID')->from(TABLE_OAUTH)
            ->where('provider')->eq($provider)
            ->beginIF($account)->andWhere('account')->in($account)->fi()
            ->fetchPairs();
    }

    /**
     * Check if user is reviewer. 
     * 
     * @param  string $account 
     * @access public
     * @return bool | array 
     */
    public function checkReviewers($account = '')
    {
        $modules = array();
        if($this->loadModel('leave', 'oa')->checkWaitingLeaves($account))       $modules[] = $this->lang->leave->common;
        if($this->loadModel('lieu', 'oa')->checkWaitingLieus($account))         $modules[] = $this->lang->lieu->common;
        if($this->loadModel('makeup', 'oa')->checkWaitingMakeups($account))     $modules[] = $this->lang->makeup->common;
        if($this->loadModel('overtime', 'oa')->checkWaitingOvertimes($account)) $modules[] = $this->lang->overtime->common;
        if($this->loadModel('refund', 'oa')->checkWaitingRefunds($account))     $modules[] = $this->lang->refund->common;
        if($modules) return array('result' => 'fail', 'message' => sprintf($this->lang->user->error->reviewer, implode(',', $modules)));
        return false;
    }

    /**
     * Create a user.
     *
     * @access public
     * @return void
     */
    public function create()
    {
        $result = parent::create();
        if($result['result'] == 'fail') return $result;

        $userID = $result['userID'];
        $user   = $this->dao->select('parent, unofficial, dept, account, realname')->from(TABLE_USER)->where('id')->eq($userID)->fetch();

        /* Add user to attend machine.*/
        $this->loadModel('iclock', 'oa')->addUser($user->account, $user->realname);

        $user->month = date('Ym');
        unset($user->realname);
        $this->dao->replace(TABLE_AMEBA_USER)->data($user)->exec();

        return $result;
    }

    /**
     * Batch create users.
     *
     * @param  int    $userLeft
     * @access public
     * @return array
     */
    public function batchCreate($userLeft)
    {
        $result = parent::batchCreate($userLeft);
        if($result['result'] == 'fail') return $result;

        $this->loadModel('iclock', 'oa');

        $month = date('Ym');
        $users = $this->dao->select('parent, unofficial, dept, account, realname')->from(TABLE_USER)->where('account')->in($this->post->account)->fetchAll();
        foreach($users as $user)
        {
            /* Add user to attend machine.*/
            $this->iclock->addUser($user->account, $user->realname);

            $user->month = $month;
            unset($user->realname);
            $this->dao->replace(TABLE_AMEBA_USER)->data($user)->exec();
        }

        return $result;
    }

    /**
     * Update a user.
     *
     * @param  string $account
     * @access public
     * @return bool
     */
    public function update($account, $from)
    {
        if(!$this->post->unofficial) $_POST['unofficial'] = '0';
        if(!$this->post->noSalary)   $_POST['noSalary']   = '0';
        if(!$this->post->dispatch)   $_POST['dispatch']   = '0';

        $result = parent::update($account, $from);
        if(!$result) return $result;

        $user = $this->dao->select('parent, unofficial, dept, account, realname, locked')->from(TABLE_USER)->where('account')->eq($account)->fetch();

        /* Update user to attend machine.*/
        if($user->locked != '2199-12-31 00:00:00') $this->loadModel('iclock', 'oa')->addUser($user->account, $user->realname);

        $user->month = date('Ym');
        unset($user->realname);
        unset($user->locked);
        $this->dao->replace(TABLE_AMEBA_USER)->data($user)->exec();

        return $result;
    }

    /**
     * Forbid a user.
     *
     * @param  string $account
     * @access public
     * @return void
     */
    public function forbid($account)
    {
        $result = parent::forbid($account);
        if(!$result) return $result;

        $this->dao->delete()->from(TABLE_AMEBA_USER)->where('month')->eq(date('Ym'))->andWhere('account')->eq($account)->exec();

        /* Delete user from attend machine. */
        $this->loadModel('iclock', 'oa')->deleteUser($account);

        return $result;
    }

    /**
     * Active a user.
     *
     * @param  string $account
     * @access public
     * @return bool
     */
    public function active($account)
    {
        $result = parent::active($account);
        if(!$result) return $result;

        $user = $this->dao->select('parent, unofficial, dept, account, realname')->from(TABLE_USER)->where('account')->eq($account)->fetch();

        /* Add user to attend machine.*/
        $this->loadModel('iclock', 'oa')->addUser($user->account, $user->realname);

        $user->month = date('Ym');
        unset($user->realname);
        $this->dao->replace(TABLE_AMEBA_USER)->data($user)->exec();

        return $result;
    }

    /**
     * Delete a user.
     *
     * @param  string    $account
     * @param  null      $id          add this param to avoid the warning of php.
     * @access public
     * @return bool
     */
    public function delete($account, $id = null) 
    {
        $result = parent::delete($account, $id);
        if(!$result) return $result;

        $this->dao->delete()->from(TABLE_AMEBA_USER)->where('month')->eq(date('Ym'))->andWhere('account')->eq($account)->exec();

        /* Delete user from attend machine. */
        $this->loadModel('iclock', 'oa')->deleteUser($account);

        return $result;
    }

    /**
     * Update token.
     * 
     * @param  string    $account 
     * @access public
     * @return bool
     */
    public function updateToken($user)
    {   
        $now                = helper::now();
        $user->token        = md5(time() . mt_rand());
        $user->tokenExpired = date('Y-m-d H:i:s', strtotime("$now +1 month"));

        $this->dao->update(TABLE_USER)
            ->set('token')->eq($user->token)
            ->set('tokenExpired')->eq($user->tokenExpired)
            ->where('account')->eq($user->account)
            ->exec();

        return $user;
    }   

    /**
     * Create login token.
     * 
     * @param  string    $account 
     * @param  string    $join 
     * @param  string    $token 
     * @access public
     * @return string
     */
    public function createLoginToken($account, $join, $token)
    {   
        return md5($account . $join . $token);
    }   

    /**
     * Login by token.
     * 
     * @param  string    $token 
     * @access public
     * @return bool|object
     */
    public function loginByToken($token)
    {
        /* First get the user from database by token. */
        $user = $this->dao->select('*')->from(TABLE_USER)
            ->where('deleted')->eq('0')
            ->andWhere('MD5(CONCAT(`account`, `join`, `token`))')->eq($token)
            ->andWhere('tokenExpired')->ge(helper::now())
            ->fetch();

        if(!$user)
        {
            $this->lang->user->loginFailed = $this->lang->user->error->tokenInvalid;
            return false;
        }

        $user = $this->identifyUser($user);

        if($user)
        {
            $groups = $this->loadModel('group')->getByAccount($user->account);
            $user->groups = array_keys($groups);
            $user->rights = $this->authorize($user);
            $this->session->set('user', $user);
            $this->app->user = $this->session->user;

            $this->loadModel('action')->create('user', $user->id, 'login');
        }

        return $user;
    }

    /**
     * Get data in JSON.
     *
     * @param  object    $user
     * @access public
     * @return array
     */
    public function getDataInJSON($user)
    {
        $data = parent::getDataInJSON($user);
        $data['user']->loginToken = $this->createLoginToken($user->account, $user->join, $user->token);

        $currentUserCompany = $this->dao->select('company')->from(TABLE_USER)->where('account')->eq($user->account)->fetch('company');
        if($currentUserCompany)
        {
            $companyName = $this->dao->select('name')->from(TABLE_COMPANY)->where('id')->eq($currentUserCompany)->fetch('name');
        }
        else
        {
            $companyName = $this->dao->select('name')->from(TABLE_COMPANY)->where('status')->eq('normal')->orderBy('major_desc,createdDate')->limit(1)->fetch('name');
        }
        $data['user']->companyName = $companyName;

        return $data;
    }

    /**
     * Create QR code.
     *
     * @access public
     * @return void
     */
    public function createQRCode()
    {
        $wwwRoot    = $this->app->getWwwRoot();
        $server     = commonModel::getSysURL() . $this->config->webRoot;
        $loginToken = $this->loadModel('user')->createLoginToken($this->app->user->account, $this->app->user->join, $this->app->user->token);
        $content    = json_encode(array('app' => 'ZDOO', 'type' => 'login', 'data' => ['token' => $loginToken, 'server' => $server]));
        $RGB        = array('236', '85', '85');
        $savePath   = $wwwRoot . "data/qrcode/";
        if(!file_exists($savePath)) @mkdir($savePath, 0777, true);
        $filePath   = $savePath . "{$this->app->user->account}.png";

        $this->app->loadClass('qrcode');
        QRcode::png($content, $filePath, 3, 6, 4, false, $RGB);

        $qrPath      = $filePath;
        $logoPath    = $wwwRoot . ($this->app->user->avatar ? $this->app->user->avatar : "theme/default/images/main/logo.png");
        $qrString    = imagecreatefromstring(file_get_contents($qrPath));
        $logoString  = imagecreatefromstring(file_get_contents($logoPath));
        $qrWidth     = imagesx($qrString);
        $qrHeight    = imagesy($qrString);
        $logoQrWidth = $qrWidth / 5;
        $logoWidth   = imagesx($logoString);
        $logoHeight  = imagesy($logoString);
        imagecopyresampled($qrString, $logoString, ($qrWidth - $logoQrWidth) / 2, ($qrHeight - $logoHeight / ($logoWidth / $logoQrWidth)) / 2, 0, 0, $logoQrWidth, $logoHeight / ($logoWidth / $logoQrWidth), $logoWidth, $logoHeight);
        imagepng($qrString, $filePath);
    }
}
